let activityModel = require('./model').activity;
let queuedItemModel = require('./model').queuedItem;
let performersModel = require('./model').performers;
let workflowMethods = require('./workflow.js');
let containerMethods = require('./container.js');
let Promise = require('bluebird');

module.exports={
    forward,
    reject,
    getDetails,
    updateActivity,
    updateQueuedItem,
    getActivitiesByPerformers,
    findActivity,
    findActivitiesByWfId,
    createQueuedItem,
    findQueuedItemsByProcessId,
    cancel,
    getQueuedItemsByPerformers,
    getQueuedItemCount,
    getPerformers
}

function getPerformers(queuedItemId){
    return new Promise(async(resolve,reject)=>{
        let queuedItem = await getDetails(queuedItemId);
        await performersModel.findById(queuedItem.performers)
        .then(r=>{
            resolve(r);
        })
        .catch(err=>{
            reject(err);
        })
    });
}

function cancel(queuedItemId){
        /**
     * 3- update container status to canceled
     * 4- delete all container meta data
     */
    return new Promise(async(resolve,reject)=>{
        try{
            let queuedItem = await getDetails(queuedItemId);
            await findQueuedItemsByProcessId(queuedItem.wfId)       
            .then(r=>{
                r.forEach(async (que)=>{
                    if(que.status !== 'completed'){
                        await containerMethods.deleteContainer(que.packages);
                        await queuedItemModel.update({status:'canceled'},{where:{id:que.id}});
                        await workflowMethods.cancel(que.wfId)
                        .then(result=>{
                            resolve(result);
                        })
                    }
                });
            })
            .catch(err=>{
                reject(err);
            })
        }
        catch(e){
            reject(e);
        }
    });

}

function findQueuedItemsByProcessId(processId){
    return new Promise((resolve,reject)=>{
        queuedItemModel.findAll({where:{wfId:processId}})
        .then(result=>{
            resolve(result);
        })
        .catch(err=>{
            reject(err);
        })
    });
}

function createQueuedItem(queuedItem_data){
    return new Promise((resolve,reject)=>{
        if(queuedItem_data.order === 1)
        queuedItem_data.status = 'inprogress';
        queuedItemModel.create(queuedItem_data)
        .then(result=>{
            resolve(result);
        })
        .catch(err=>{
            reject(err);
        })
    });
}

function findActivity(activityId){
    return new Promise((resolve,reject)=>{
        activityModel.findById(activityId)
        .then(result=>{
            resolve(result);
        })
        .catch(err=>{
            reject(err);
        });
    });
}

function findActivitiesByWfId(wfId){
    return new Promise((resolve,reject)=>{
        activityModel.findAll({where:{wfId:wfId}})
        .then(result=>{
            resolve(result);
        })
        .catch(err=>{
            reject(err);
        })
    })
}

function getQueuedItemsByPerformers(performersId){
    return new Promise(
        (resolve,reject)=>{
        queuedItemModel.findAll({where:
            {
                $and:[
                    {
                        status:'inprogress'
                    },
                    {
                        performers:performersId
                    }
                ]
            }
        })
        .then(result=>{
            resolve(result);
        })
        .catch(err=>{
            reject(err);
        });
    });
}   

function getActivitiesByPerformers(performersId){
    /**
     * get activities by performers for user to add action (forward,reject or cancel)
     */
    return new Promise((resolve,reject)=>{
        queuedItemModel.findAll({where:{performers:performersId}})
        .then(r=>{
            resolve(r);
        })
        .catch(err=>{
            reject(err);
        });
    });
}
//forward activity in 3 conditions if order 1 which is beginning of the workflow
//or between beginning and last index of activities in workflow
//or last index of activities in the workflow 

function forward(queuedItemId, data){
    return new Promise(async(resolve,reject)=>{
        let order = await checkQueuedItemOrder(queuedItemId);
        let queuedItem = await getDetails(queuedItemId);
        let length = await getQueuedItemCount(queuedItem.wfId);

        if(order == 1)
        {
            let container = {
                name: data.name,
                data: data
            }
            containerMethods.create(container)
            .then(async r=>{
                try{
                    let nextQueuedItem = await queuedItemModel.find({where:{order:queuedItem.order+1}});
                    await queuedItemModel.update({status:'completed',packages:r.id},{where:{id:queuedItemId}});
                    await queuedItemModel.update({status:'inprogress',packages:r.id},{where:{id:nextQueuedItem.id}})
                    .then(result=>{
                        resolve(result);
                    })
                    .catch(err=>{
                        reject(err);
                    })
                }
                catch(e){
                    reject(e);
                }
            })
            .catch(err=>{
                reject(err);
            })
        }
        else 
        {
            if(order < length)
            {
               
                    try{
                        await containerMethods.update(queuedItem.packages,data);
                        let nextQueuedItem = await queuedItemModel.find({where:{order:queuedItem.order+1}});
                        await queuedItemModel.update({status:'completed'},{where:{id:queuedItemId}});
                        await queuedItemModel.update({status:'inprogress',packages:queuedItem.packages},{where:{id:nextQueuedItem.id}})
                        .then(result=>{
                            resolve(result);
                        })
                        .catch(err=>{
                            reject(err);
                        });
                    }
                    catch(e){
                        reject(e);
                    }
            }
            else 
            {
                try{
                    queuedItemModel.update({status:'completed'},{where:{id:queuedItemId}});
                    let s = require('./workflow.js');
                    s.complete(queuedItem.wfId)
                    .then(result=>{
                        resolve(result);
                    })
                    .catch(err=>{
                        reject(err);
                    })
                }
                catch(e){
                    reject(e);
                }
            }
        }
    });
}

function checkQueuedItemOrder(QueuedItemId){
    return new Promise((resolve,reject)=>{
        queuedItemModel.findById(QueuedItemId)
        .then(r=>{
            resolve(r.order);
        })
        .catch(err=>{
            reject(err);
        })
    });
}
function reject(queuedItemId){
    /*
        1- check if its not the first activity inthat case there are no reject will cancel the whole workflow
        2- container.getDetails
        3- container.update with the last activity.getDetails
        4- change current activity.update(status) to dormant and last activity to inProgress
     */
    return new Promise(async(resolve,reject)=>{
        let queuedItem = await getDetails(queuedItemId);
        if(queuedItem.order === 1 || queuedItem.status === 'rejected')
            reject('ERROR: cannot reject queuedItem either first item or already rejected');
        await updateQueuedItem(queuedItemId,{status:'rejected'});
        await updateQueuedItem(queuedItemId-1,{status:'inprogress'})
        .then(r=>{
            resolve(r);
        })
        .catch(err=>{
            reject(err);
        })
    });
}
function getQueuedItemCount(processId){
    /**
     * check from workflow instance(process) the current count of queuedItems
     */
    return new Promise((resolve,reject)=>{
        queuedItemModel.findAll({where:{wfId:processId}})
        .then(r=>{
            console.log(r);
            resolve(r.length);
        })
        .catch(err=>{
            reject(err);
        })
    });
}
function getDetails(queuedItemId){
    return new Promise((resolve,reject)=>{
        queuedItemModel.findById(queuedItemId)
        .then(r=>{
            resolve(r);
        })
        .catch(err=>{
            reject(err);
        })
    });
}
function updateActivity(){
    return null;
}

function updateQueuedItem(queuedItemId, data){
    /**
     * 1- update activity details usually status and workflow to process (template to implementation)
     */
    return new Promise((resolve,reject)=>{
        queuedItemModel.findById(queuedItemId)
        .then(r=>{
            if(!r)
            reject('queuedItemId incorrect or not found');
            queuedItemModel.update(data,{where:{id:queuedItemId}})
            .then(result=>{
                resolve(result);
            })
            .catch(err=>{
                reject(err);
            })
        })
        .catch(err=>{
            reject(err);
        })
    });
}

function updateQueuedItem(queuedItemId, data){
    return new Promise((resolve,reject)=>{
        queuedItemModel.update(data,{where:{id:queuedItemId}})
        .then(result=>{
            resolve(result);
        })
        .catch(err=>{
            reject(err);
        });
    });
}