let workflowModel = require('./model').workflow;
let processModel = require('./model').process;
let activMethods = require('./activity.js');
let Promise = require('bluebird');

module.exports={
    initiate,
    cancel,
    complete,
    getDetails,
    update,
    findWorkflowById,
    createProcess
}

function initiate(wfId){
    return new Promise(async(resolve,reject)=>{
        //create new entry for process and queuedItems
        let processData = {};
        try{
            let workflow_data = await findWorkflowById(wfId);
            processData = {
               name: workflow_data.name,
               workflow_template: wfId,
               status: 'inprogress'
            }

            processData = await createProcess(processData);
            await createQueuedItems(wfId,processData.id);
            resolve(processData);
        }
        catch(e){
            reject(e);
        }
    });
}

function findWorkflowById(wfId){
    return new Promise((resolve,reject)=>{
        workflowModel.findById(wfId)
        .then(findResult=>{
            let process = {
                name:findResult.name,
                status:'inProgress',
            }
            resolve(process);
        })
        .catch(err=>{
            reject(err);
        });
    });
}

function createProcess(process_data){
    return new Promise((resolve,reject)=>{
        processModel.create(process_data)
        .then(creationResult=>{
            process_data.id = creationResult.id;
            resolve(process_data);
        })
        .catch(err=>{
            reject(err);
        })
    })
}

function createQueuedItems(wfId,processId){
    return new Promise((resolve,reject)=>{
        activMethods.findActivitiesByWfId(wfId)
        .then(activities=>{
            activities.forEach(async (activityItem)=>{
                let act = await activMethods.createQueuedItem({
                   name:activityItem.name,
                   order:activityItem.order,
                   status:activityItem.status,
                   wfId:processId,
                   acl:activityItem.acl,
                   performers:activityItem.performers,
                   methods:activityItem.methods
                });
                resolve(act);
            })
        })
        .catch(err=>{
            reject(err);
        })
    });
}



function cancel(processId){

    return new Promise((resolve,reject)=>{
        //update process status to canceled
        try{
            processModel.findById(processId)
            .then(async processResult=>{
                if(!processResult)
                reject('process not found or wrong ID');
                if(processResult.status === 'completed')
                reject('process cannot be canceled');
                else {
                    await update(processId,{status:'canceled'});
                    let queuedItems = await activMethods.findQueuedItemsByProcessId(processId);
                    queuedItems.forEach(async(queueditem)=>{
                        await activMethods.updateQueuedItem(queueditem.id,{status:'canceled'});
                    });
                    resolve('process canceled');
                }
            })
            .catch(err=>{
                reject(err);
            })
            
        }
        catch(err){
            reject(err);
        }
    });
}

function complete(processId){
    return new Promise((resolve,reject)=>{
            processModel.findById(processId)
            .then(processResult=>{
                if(!processResult)
                reject('process not found or wrong ID');
                if(processResult.status === 'canceled')
                reject('process cannot be completed');
                else {
                    update(processId,{status:'completed'})
                    .then(result=>{
                        resolve(result);
                    })
                    .catch(err=>{
                        reject(err);
                    })
                }
            })
            .catch(err=>{
                reject(err);
            })
    });
}

function update(processId,data){
    /**
     * 1- update process details in normal case just status (dormant, inProgress, canceled, completed)
     */
    return new Promise((resolve,reject)=>{
        processModel.update(data,{where:{id:processId}})
        .then(result=>{
            resolve(result);
        })
        .catch(err=>{
            reject(err);
        })
    });
    
}

function getDetails(processId){
    /**
     * 1- get process details query on process to get process details for activities and container
     */
    return new Promise((resolve,reject)=>{
        processModel.findById(processId)
        .then(r=>{
            resolve(r);
        })
        .catch(err=>{
            reject(err);
        })
    });
}